@@ -1,20 +1,24 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/memory/ref_counted.h"
 
+#include <type_traits>
 #include <utility>
 
+#include "base/memory/raw_ptr.h"
 #include "base/test/gtest_util.h"
-#include "base/test/opaque_ref_counted.h"
+#include "build/build_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+namespace base {
+namespace subtle {
 namespace {
 
 class SelfAssign : public base::RefCounted<SelfAssign> {
  protected:
-  virtual ~SelfAssign() {}
+  virtual ~SelfAssign() = default;
 
  private:
   friend class base::RefCounted<SelfAssign>;
@@ -22,7 +26,7 @@ class SelfAssign : public base::RefCounted<SelfAssign> {
 
 class Derived : public SelfAssign {
  protected:
-  ~Derived() override {}
+  ~Derived() override = default;
 
  private:
   friend class base::RefCounted<Derived>;
@@ -112,9 +116,29 @@ class Other : public base::RefCounted<Other> {
  private:
   friend class base::RefCounted<Other>;
 
-  ~Other() {}
+  ~Other() = default;
 };
 
+class HasPrivateDestructorWithDeleter;
+
+struct Deleter {
+  static void Destruct(const HasPrivateDestructorWithDeleter* x);
+};
+
+class HasPrivateDestructorWithDeleter
+    : public base::RefCounted<HasPrivateDestructorWithDeleter, Deleter> {
+ public:
+  HasPrivateDestructorWithDeleter() = default;
+
+ private:
+  friend struct Deleter;
+  ~HasPrivateDestructorWithDeleter() = default;
+};
+
+void Deleter::Destruct(const HasPrivateDestructorWithDeleter* x) {
+  delete x;
+}
+
 scoped_refptr<Other> Overloaded(scoped_refptr<Other> other) {
   return other;
 }
@@ -127,19 +151,44 @@ class InitialRefCountIsOne : public base::RefCounted<InitialRefCountIsOne> {
  public:
   REQUIRE_ADOPTION_FOR_REFCOUNTED_TYPE();
 
-  InitialRefCountIsOne() {}
+  InitialRefCountIsOne() = default;
 
  private:
   friend class base::RefCounted<InitialRefCountIsOne>;
-  ~InitialRefCountIsOne() {}
+  ~InitialRefCountIsOne() = default;
 };
 
-}  // end namespace
+// Checks that the scoped_refptr is null before the reference counted object is
+// destroyed.
+class CheckRefptrNull : public base::RefCounted<CheckRefptrNull> {
+ public:
+  // Set the last scoped_refptr that will have a reference to this object.
+  void set_scoped_refptr(scoped_refptr<CheckRefptrNull>* ptr) { ptr_ = ptr; }
+
+ protected:
+  virtual ~CheckRefptrNull() {
+    EXPECT_NE(ptr_, nullptr);
+    EXPECT_EQ(ptr_->get(), nullptr);
+  }
+
+ private:
+  friend class base::RefCounted<CheckRefptrNull>;
+
+  raw_ptr<scoped_refptr<CheckRefptrNull>> ptr_ = nullptr;
+};
+
+}  // namespace
 
 TEST(RefCountedUnitTest, TestSelfAssignment) {
   SelfAssign* p = new SelfAssign;
   scoped_refptr<SelfAssign> var(p);
-  var = var;
+  var = *&var;  // The *& defeats Clang's -Wself-assign warning.
+  EXPECT_EQ(var.get(), p);
+  var = std::move(var);
+  EXPECT_EQ(var.get(), p);
+  var.swap(var);
+  EXPECT_EQ(var.get(), p);
+  swap(var, var);
   EXPECT_EQ(var.get(), p);
 }
 
@@ -168,37 +217,6 @@ TEST(RefCountedUnitTest, ScopedRefPtrToSelfMoveAssignment) {
   EXPECT_TRUE(ScopedRefPtrToSelf::was_destroyed());
 }
 
-TEST(RefCountedUnitTest, ScopedRefPtrToOpaque) {
-  scoped_refptr<base::OpaqueRefCounted> initial = base::MakeOpaqueRefCounted();
-  base::TestOpaqueRefCounted(initial);
-
-  scoped_refptr<base::OpaqueRefCounted> assigned;
-  assigned = initial;
-
-  scoped_refptr<base::OpaqueRefCounted> copied(initial);
-
-  scoped_refptr<base::OpaqueRefCounted> moved(std::move(initial));
-
-  scoped_refptr<base::OpaqueRefCounted> move_assigned;
-  move_assigned = std::move(moved);
-}
-
-TEST(RefCountedUnitTest, ScopedRefPtrToOpaqueThreadSafe) {
-  scoped_refptr<base::OpaqueRefCountedThreadSafe> initial =
-      base::MakeOpaqueRefCountedThreadSafe();
-  base::TestOpaqueRefCountedThreadSafe(initial);
-
-  scoped_refptr<base::OpaqueRefCountedThreadSafe> assigned;
-  assigned = initial;
-
-  scoped_refptr<base::OpaqueRefCountedThreadSafe> copied(initial);
-
-  scoped_refptr<base::OpaqueRefCountedThreadSafe> moved(std::move(initial));
-
-  scoped_refptr<base::OpaqueRefCountedThreadSafe> move_assigned;
-  move_assigned = std::move(moved);
-}
-
 TEST(RefCountedUnitTest, BooleanTesting) {
   scoped_refptr<SelfAssign> ptr_to_an_instance = new SelfAssign;
   EXPECT_TRUE(ptr_to_an_instance);
@@ -232,10 +250,38 @@ TEST(RefCountedUnitTest, Equality) {
   scoped_refptr<SelfAssign> p2(new SelfAssign);
 
   EXPECT_EQ(p1, p1);
+  EXPECT_EQ(p1.get(), p1);
+  EXPECT_EQ(p1, p1.get());
+
   EXPECT_EQ(p2, p2);
+  EXPECT_EQ(p2.get(), p2);
+  EXPECT_EQ(p2, p2.get());
 
   EXPECT_NE(p1, p2);
+  EXPECT_NE(p1.get(), p2);
+  EXPECT_NE(p1, p2.get());
+
   EXPECT_NE(p2, p1);
+  EXPECT_NE(p2.get(), p1);
+  EXPECT_NE(p2, p1.get());
+}
+
+TEST(RefCountedUnitTest, Ordering) {
+  scoped_refptr<SelfAssign> p1(new SelfAssign);
+  scoped_refptr<SelfAssign> p2(new SelfAssign);
+  EXPECT_NE(p1, p2);
+
+  if (p1.get() > p2.get()) {
+    p1.swap(p2);
+  }
+
+  EXPECT_LT(p1, p2);
+  EXPECT_LT(p1.get(), p2);
+  EXPECT_LT(p1, p2.get());
+
+  EXPECT_GT(p2, p1);
+  EXPECT_GT(p2.get(), p1);
+  EXPECT_GT(p2, p1.get());
 }
 
 TEST(RefCountedUnitTest, NullptrEquality) {
@@ -248,6 +294,13 @@ TEST(RefCountedUnitTest, NullptrEquality) {
   EXPECT_EQ(ptr_to_nullptr, nullptr);
 }
 
+TEST(RefCountedUnitTest, NullptrOrdering) {
+  scoped_refptr<SelfAssign> ptr_to_an_instance(new SelfAssign);
+
+  EXPECT_LT(nullptr, ptr_to_an_instance);
+  EXPECT_GT(ptr_to_an_instance, nullptr);
+}
+
 TEST(RefCountedUnitTest, ConvertibleEquality) {
   scoped_refptr<Derived> p1(new Derived);
   scoped_refptr<SelfAssign> p2;
@@ -543,47 +596,213 @@ TEST(RefCountedUnitTest, MoveConstructorDerived) {
 }
 
 TEST(RefCountedUnitTest, TestOverloadResolutionCopy) {
-  scoped_refptr<Derived> derived(new Derived);
-  scoped_refptr<SelfAssign> expected(derived);
+  const scoped_refptr<Derived> derived(new Derived);
+  const scoped_refptr<SelfAssign> expected(derived);
   EXPECT_EQ(expected, Overloaded(derived));
 
-  scoped_refptr<Other> other(new Other);
+  const scoped_refptr<Other> other(new Other);
   EXPECT_EQ(other, Overloaded(other));
 }
 
 TEST(RefCountedUnitTest, TestOverloadResolutionMove) {
   scoped_refptr<Derived> derived(new Derived);
-  scoped_refptr<SelfAssign> expected(derived);
+  const scoped_refptr<SelfAssign> expected(derived);
   EXPECT_EQ(expected, Overloaded(std::move(derived)));
 
   scoped_refptr<Other> other(new Other);
-  scoped_refptr<Other> other2(other);
+  const scoped_refptr<Other> other2(other);
   EXPECT_EQ(other2, Overloaded(std::move(other)));
 }
 
+TEST(RefCountedUnitTest, TestMakeRefCounted) {
+  scoped_refptr<Derived> derived = new Derived;
+  EXPECT_TRUE(derived->HasOneRef());
+  derived.reset();
+
+  scoped_refptr<Derived> derived2 = base::MakeRefCounted<Derived>();
+  EXPECT_TRUE(derived2->HasOneRef());
+  derived2.reset();
+}
+
 TEST(RefCountedUnitTest, TestInitialRefCountIsOne) {
   scoped_refptr<InitialRefCountIsOne> obj =
       base::MakeRefCounted<InitialRefCountIsOne>();
   EXPECT_TRUE(obj->HasOneRef());
-  obj = nullptr;
+  obj.reset();
 
   scoped_refptr<InitialRefCountIsOne> obj2 =
       base::AdoptRef(new InitialRefCountIsOne);
   EXPECT_TRUE(obj2->HasOneRef());
-  obj2 = nullptr;
+  obj2.reset();
 
   scoped_refptr<Other> obj3 = base::MakeRefCounted<Other>();
   EXPECT_TRUE(obj3->HasOneRef());
-  obj3 = nullptr;
+  obj3.reset();
+}
+
+TEST(RefCountedUnitTest, TestPrivateDestructorWithDeleter) {
+  // Ensure that RefCounted doesn't need the access to the pointee dtor when
+  // a custom deleter is given.
+  scoped_refptr<HasPrivateDestructorWithDeleter> obj =
+      base::MakeRefCounted<HasPrivateDestructorWithDeleter>();
+}
+
+TEST(RefCountedUnitTest, TestReset) {
+  ScopedRefPtrCountBase::reset_count();
+
+  // Create ScopedRefPtrCountBase that is referenced by |obj1| and |obj2|.
+  scoped_refptr<ScopedRefPtrCountBase> obj1 =
+      base::MakeRefCounted<ScopedRefPtrCountBase>();
+  scoped_refptr<ScopedRefPtrCountBase> obj2 = obj1;
+  EXPECT_NE(obj1.get(), nullptr);
+  EXPECT_NE(obj2.get(), nullptr);
+  EXPECT_EQ(ScopedRefPtrCountBase::constructor_count(), 1);
+  EXPECT_EQ(ScopedRefPtrCountBase::destructor_count(), 0);
+
+  // Check that calling reset() on |obj1| resets it. |obj2| still has a
+  // reference to the ScopedRefPtrCountBase so it shouldn't be reset.
+  obj1.reset();
+  EXPECT_EQ(obj1.get(), nullptr);
+  EXPECT_EQ(ScopedRefPtrCountBase::constructor_count(), 1);
+  EXPECT_EQ(ScopedRefPtrCountBase::destructor_count(), 0);
+
+  // Check that calling reset() on |obj2| resets it and causes the deletion of
+  // the ScopedRefPtrCountBase.
+  obj2.reset();
+  EXPECT_EQ(obj2.get(), nullptr);
+  EXPECT_EQ(ScopedRefPtrCountBase::constructor_count(), 1);
+  EXPECT_EQ(ScopedRefPtrCountBase::destructor_count(), 1);
+}
+
+TEST(RefCountedUnitTest, TestResetAlreadyNull) {
+  // Check that calling reset() on a null scoped_refptr does nothing.
+  scoped_refptr<ScopedRefPtrCountBase> obj;
+  obj.reset();
+  // |obj| should still be null after calling reset().
+  EXPECT_EQ(obj.get(), nullptr);
+}
+
+TEST(RefCountedUnitTest, TestResetByNullptrAssignment) {
+  // Check that assigning nullptr resets the object.
+  auto obj = base::MakeRefCounted<ScopedRefPtrCountBase>();
+  EXPECT_NE(obj.get(), nullptr);
+
+  obj = nullptr;
+  EXPECT_EQ(obj.get(), nullptr);
+}
+
+TEST(RefCountedUnitTest, CheckScopedRefptrNullBeforeObjectDestruction) {
+  scoped_refptr<CheckRefptrNull> obj = base::MakeRefCounted<CheckRefptrNull>();
+  obj->set_scoped_refptr(&obj);
+
+  // Check that when reset() is called the scoped_refptr internal pointer is set
+  // to null before the reference counted object is destroyed. This check is
+  // done by the CheckRefptrNull destructor.
+  obj.reset();
+  EXPECT_EQ(obj.get(), nullptr);
 }
 
 TEST(RefCountedDeathTest, TestAdoptRef) {
-  EXPECT_DCHECK_DEATH(make_scoped_refptr(new InitialRefCountIsOne));
+  // Check that WrapRefCounted() DCHECKs if passed a type that defines
+  // REQUIRE_ADOPTION_FOR_REFCOUNTED_TYPE.
+  EXPECT_DCHECK_DEATH(base::WrapRefCounted(new InitialRefCountIsOne));
 
+  // Check that AdoptRef() DCHECKs if passed a nullptr.
   InitialRefCountIsOne* ptr = nullptr;
   EXPECT_DCHECK_DEATH(base::AdoptRef(ptr));
 
+  // Check that AdoptRef() DCHECKs if passed an object that doesn't need to be
+  // adopted.
   scoped_refptr<InitialRefCountIsOne> obj =
       base::MakeRefCounted<InitialRefCountIsOne>();
   EXPECT_DCHECK_DEATH(base::AdoptRef(obj.get()));
 }
+
+#if defined(ARCH_CPU_64_BITS)
+class RefCountedOverflowTest : public ::testing::Test {
+ public:
+  static uint32_t& GetMutableRefCount(RefCountedBase* ref_counted) {
+    return ref_counted->ref_count_;
+  }
+
+  static std::atomic_int& GetMutableRefCount(
+      RefCountedThreadSafeBase* ref_counted) {
+    return ref_counted->ref_count_.ref_count_;
+  }
+};
+
+TEST_F(RefCountedOverflowTest, NonThreadSafeStartFromZero) {
+  class Overflow : public base::RefCounted<Overflow> {
+   public:
+    Overflow() { EXPECT_FALSE(HasOneRef()); }
+
+   private:
+    friend class base::RefCounted<Overflow>;
+    ~Overflow() = default;
+  };
+
+  auto p = base::MakeRefCounted<Overflow>();
+  GetMutableRefCount(p.get()) = std::numeric_limits<uint32_t>::max();
+  EXPECT_CHECK_DEATH(p->AddRef());
+  // Ensure `p` doesn't leak and fail lsan builds.
+  GetMutableRefCount(p.get()) = 1;
+}
+
+TEST_F(RefCountedOverflowTest, NonThreadSafeStartFromOne) {
+  class Overflow : public base::RefCounted<Overflow> {
+   public:
+    REQUIRE_ADOPTION_FOR_REFCOUNTED_TYPE();
+
+    Overflow() { EXPECT_TRUE(HasOneRef()); }
+
+   private:
+    friend class base::RefCounted<Overflow>;
+    ~Overflow() = default;
+  };
+
+  auto p = base::MakeRefCounted<Overflow>();
+  GetMutableRefCount(p.get()) = std::numeric_limits<uint32_t>::max();
+  EXPECT_CHECK_DEATH(p->AddRef());
+  // Ensure `p` doesn't leak and fail lsan builds.
+  GetMutableRefCount(p.get()) = 1;
+}
+
+TEST_F(RefCountedOverflowTest, ThreadSafeStartFromZero) {
+  class Overflow : public base::RefCountedThreadSafe<Overflow> {
+   public:
+    Overflow() { EXPECT_FALSE(HasOneRef()); }
+
+   private:
+    friend class base::RefCountedThreadSafe<Overflow>;
+    ~Overflow() = default;
+  };
+
+  auto p = base::MakeRefCounted<Overflow>();
+  GetMutableRefCount(p.get()) = std::numeric_limits<int>::max();
+  EXPECT_CHECK_DEATH(p->AddRef());
+  // Ensure `p` doesn't leak and fail lsan builds.
+  GetMutableRefCount(p.get()) = 1;
+}
+
+TEST_F(RefCountedOverflowTest, ThreadSafeStartFromOne) {
+  class Overflow : public base::RefCountedThreadSafe<Overflow> {
+   public:
+    REQUIRE_ADOPTION_FOR_REFCOUNTED_TYPE();
+
+    Overflow() { EXPECT_TRUE(HasOneRef()); }
+
+   private:
+    friend class base::RefCountedThreadSafe<Overflow>;
+    ~Overflow() = default;
+  };
+
+  auto p = base::MakeRefCounted<Overflow>();
+  GetMutableRefCount(p.get()) = std::numeric_limits<int>::max();
+  EXPECT_CHECK_DEATH(p->AddRef());
+  // Ensure `p` doesn't leak and fail lsan builds.
+  GetMutableRefCount(p.get()) = 1;
+}
+#endif
+
+}  // namespace subtle
+}  // namespace base
