@@ -1,4 +1,4 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,8 @@
 
 #include <utility>
 
+#include "base/check_op.h"
 #include "base/feature_list.h"
-#include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/task/task_features.h"
 #include "base/task/thread_pool/task_tracker.h"
@@ -15,6 +15,8 @@
 namespace base {
 namespace internal {
 
+ExecutionEnvironment::~ExecutionEnvironment() = default;
+
 TaskSource::Transaction::Transaction(TaskSource* task_source)
     : task_source_(task_source) {
   task_source->lock_.Acquire();
@@ -27,49 +29,64 @@ TaskSource::Transaction::Transaction(TaskSource::Transaction&& other)
 
 TaskSource::Transaction::~Transaction() {
   if (task_source_) {
-    task_source_->lock_.AssertAcquired();
-    task_source_->lock_.Release();
+    Release();
   }
 }
 
-SequenceSortKey TaskSource::Transaction::GetSortKey() const {
-  return task_source_->GetSortKey();
-}
-
 void TaskSource::Transaction::UpdatePriority(TaskPriority priority) {
-  if (FeatureList::IsEnabled(kAllTasksUserBlocking))
-    return;
   task_source_->traits_.UpdatePriority(priority);
   task_source_->priority_racy_.store(task_source_->traits_.priority(),
                                      std::memory_order_relaxed);
 }
 
-void TaskSource::SetHeapHandle(const HeapHandle& handle) {
-  heap_handle_ = handle;
+void TaskSource::Transaction::Release() NO_THREAD_SAFETY_ANALYSIS {
+  DCHECK(task_source_);
+  task_source_->lock_.AssertAcquired();
+  task_source_->lock_.Release();
+  task_source_ = nullptr;
 }
 
-void TaskSource::ClearHeapHandle() {
-  heap_handle_ = HeapHandle();
+void TaskSource::SetImmediateHeapHandle(const HeapHandle& handle) {
+  immediate_pq_heap_handle_ = handle;
+}
+
+void TaskSource::ClearImmediateHeapHandle() {
+  immediate_pq_heap_handle_ = HeapHandle();
+}
+
+void TaskSource::SetDelayedHeapHandle(const HeapHandle& handle) {
+  delayed_pq_heap_handle_ = handle;
+}
+
+void TaskSource::ClearDelayedHeapHandle() {
+  delayed_pq_heap_handle_ = HeapHandle();
 }
 
 TaskSource::TaskSource(const TaskTraits& traits,
-                       TaskRunner* task_runner,
                        TaskSourceExecutionMode execution_mode)
     : traits_(traits),
       priority_racy_(traits.priority()),
-      task_runner_(task_runner),
-      execution_mode_(execution_mode) {
-  DCHECK(task_runner_ ||
-         execution_mode_ == TaskSourceExecutionMode::kParallel ||
-         execution_mode_ == TaskSourceExecutionMode::kJob);
-}
+      execution_mode_(execution_mode) {}
 
-TaskSource::~TaskSource() = default;
+TaskSource::~TaskSource() {
+  // If this fails, a Transaction was likely held while releasing a reference to
+  // its associated task source, which lead to its destruction. Owners of
+  // Transaction must ensure to hold onto a reference of the associated task
+  // source at least until the Transaction is released to prevent UAF.
+  lock_.AssertNotHeld();
+}
 
 TaskSource::Transaction TaskSource::BeginTransaction() {
   return Transaction(this);
 }
 
+void TaskSource::ClearForTesting() {
+  auto task = Clear(nullptr);
+  if (task) {
+    std::move(task->task).Run();
+  }
+}
+
 RegisteredTaskSource::RegisteredTaskSource() = default;
 
 RegisteredTaskSource::RegisteredTaskSource(std::nullptr_t)
@@ -134,7 +151,8 @@ Task RegisteredTaskSource::TakeTask(TaskSource::Transaction* transaction) {
   return task_source_->TakeTask(transaction);
 }
 
-Task RegisteredTaskSource::Clear(TaskSource::Transaction* transaction) {
+std::optional<Task> RegisteredTaskSource::Clear(
+    TaskSource::Transaction* transaction) {
   DCHECK(!transaction || transaction->task_source() == get());
   return task_source_->Clear(transaction);
 }
@@ -149,12 +167,21 @@ bool RegisteredTaskSource::DidProcessTask(
   return task_source_->DidProcessTask(transaction);
 }
 
+bool RegisteredTaskSource::WillReEnqueue(TimeTicks now,
+                                         TaskSource::Transaction* transaction) {
+  DCHECK(!transaction || transaction->task_source() == get());
+#if DCHECK_IS_ON()
+  DCHECK_EQ(State::kInitial, run_step_);
+#endif  // DCHECK_IS_ON()
+  return task_source_->WillReEnqueue(now, transaction);
+}
+
 RegisteredTaskSource::RegisteredTaskSource(
     scoped_refptr<TaskSource> task_source,
     TaskTracker* task_tracker)
     : task_source_(std::move(task_source)), task_tracker_(task_tracker) {}
 
-TransactionWithRegisteredTaskSource::TransactionWithRegisteredTaskSource(
+RegisteredTaskSourceAndTransaction::RegisteredTaskSourceAndTransaction(
     RegisteredTaskSource task_source_in,
     TaskSource::Transaction transaction_in)
     : task_source(std::move(task_source_in)),
@@ -163,12 +190,33 @@ TransactionWithRegisteredTaskSource::TransactionWithRegisteredTaskSource(
 }
 
 // static:
-TransactionWithRegisteredTaskSource
-TransactionWithRegisteredTaskSource::FromTaskSource(
+RegisteredTaskSourceAndTransaction
+RegisteredTaskSourceAndTransaction::FromTaskSource(
     RegisteredTaskSource task_source_in) {
   auto transaction = task_source_in->BeginTransaction();
-  return TransactionWithRegisteredTaskSource(std::move(task_source_in),
-                                             std::move(transaction));
+  return RegisteredTaskSourceAndTransaction(std::move(task_source_in),
+                                            std::move(transaction));
+}
+
+TaskSourceAndTransaction::TaskSourceAndTransaction(
+    TaskSourceAndTransaction&& other) = default;
+
+TaskSourceAndTransaction::~TaskSourceAndTransaction() = default;
+
+TaskSourceAndTransaction::TaskSourceAndTransaction(
+    scoped_refptr<TaskSource> task_source_in,
+    TaskSource::Transaction transaction_in)
+    : task_source(std::move(task_source_in)),
+      transaction(std::move(transaction_in)) {
+  DCHECK_EQ(task_source.get(), transaction.task_source());
+}
+
+// static:
+TaskSourceAndTransaction TaskSourceAndTransaction::FromTaskSource(
+    scoped_refptr<TaskSource> task_source_in) {
+  auto transaction = task_source_in->BeginTransaction();
+  return TaskSourceAndTransaction(std::move(task_source_in),
+                                  std::move(transaction));
 }
 
 }  // namespace internal
