@@ -1,4 +1,4 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,14 +14,13 @@ namespace cc {
 // The data is captured in LocalFrameViewUKMAggregator and passed back through
 // the proxy when a main frame ends. LayerTreeHost updates the update_layers_
 // value in LayerTreeHost::UpdateLayers.
-// TODO(schenney): Include work done in LayerTreeHost::AnimateLayers?
 struct CC_EXPORT BeginMainFrameMetrics {
   base::TimeDelta handle_input_events;
   base::TimeDelta animate;
   base::TimeDelta style_update;
   base::TimeDelta layout_update;
+  base::TimeDelta accessibility;
   base::TimeDelta prepaint;
-  base::TimeDelta compositing_assignments;
   base::TimeDelta compositing_inputs;
   base::TimeDelta paint;
   base::TimeDelta composite_commit;
