@@ -1,4 +1,4 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,14 +17,16 @@ WindowManagerHandler::~WindowManagerHandler() = default;
 
 protocol::Response WindowManagerHandler::EnterOverviewMode() {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-  bool success = ash::Shell::Get()->overview_controller()->StartOverview();
-  return success ? protocol::Response::OK()
-                 : protocol::Response::Error("Overview failed");
+  bool success = ash::Shell::Get()->overview_controller()->StartOverview(
+      ash::OverviewStartAction::kDevTools);
+  return success ? protocol::Response::Success()
+                 : protocol::Response::ServerError("Overview failed");
 }
 
 protocol::Response WindowManagerHandler::ExitOverviewMode() {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-  bool success = ash::Shell::Get()->overview_controller()->EndOverview();
-  return success ? protocol::Response::OK()
-                 : protocol::Response::Error("Overview failed");
+  bool success = ash::Shell::Get()->overview_controller()->EndOverview(
+      ash::OverviewEndAction::kDevTools);
+  return success ? protocol::Response::Success()
+                 : protocol::Response::ServerError("Overview failed");
 }
